#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#define START_LINE 400
#define END_LINE 450
#define NUM_REL_LINES (END_LINE-START_LINE + 1)
#define NUM_FREQZ 25

//Sound generation
volatile unsigned long accumulator;
unsigned char highbyte;
unsigned long increment;
char sineTable[256];
int cur_pos;
unsigned long note_inkz[NUM_FREQZ];

//Video scanning
unsigned char lineDone[NUM_REL_LINES];
unsigned char puckLine[NUM_REL_LINES];
unsigned char noteLine[NUM_REL_LINES];
int puckMed;
int noteMed;
int medFlag;
int diff;

// Video signal
volatile int linecount; // current line (of 262)
volatile int linepos; // horizontal position on line
volatile int framecount;

unsigned int i;

//timer 0 overflow ISR
ISR (TIMER0_OVF_vect)
{
 //iterate through sine table to generate sine wave
 accumulator = accumulator + increment;
 highbyte = (char)(accumulator >> 24);
 OCR0B = 128 + sineTable[highbyte];
}

//VSYNC (Pin D.2, red Wire)
ISR (INT0_vect)
{
 //reset linecount
 //if we are on odd line, start at 1
 if (PIND & (1 << PIND4))
       linecount = 1;
       else
       linecount = 0;
}

//BURST (Pin D.3, Blue Wire)
ISR (INT1_vect)
{
 //on new line, increment line count by 2 (interlaced)
 linecount+=2;
}


void init(void) {

 //sound stuff
 // Setup timer0 for PWM
 TCCR0B = 2; //prescalar of 8
 TIMSK0 = (1<<TOIE0); // overflow interrupt
 TCCR0A = (1<<COM0B0) | (1<<COM0B1) | (1<<WGM00) | (1<<WGM01); //fast PWM, toggle OCOB (B.4)
 DDRB = (1<<PINB4); // B.4 an output, pwm.
 increment = 115066967L; 
 // increment = 2^32*256*Fout/20e6 = 439808 * Fout
 // init the note table
 note_inkz[0] = 115066967L;
 note_inkz[1] = 118486474L;
 note_inkz[2] = 121905981L;
 note_inkz[3] = 125529999L;
 note_inkz[4] = 129154017L;
 note_inkz[5] = 132995740L;
 note_inkz[6] = 136837463L;
 note_inkz[7] = 140905687L;
 note_inkz[8] = 144973911L;
 note_inkz[9] = 149284029L;
 note_inkz[10] = 153594147L;
 note_inkz[11] = 158159354L;
 note_inkz[12] = 162724561L;
 note_inkz[13] = 167564648L;
 note_inkz[14] = 172404736L;
 note_inkz[15] = 177528499L;
 note_inkz[16] = 182652262L;
 note_inkz[17] = 188083891L;
 note_inkz[18] = 193515520L;
 note_inkz[19] = 199268208L;
 note_inkz[20] = 205020897L;
 note_inkz[21] = 211116636L;
 note_inkz[22] = 217212375L;
 note_inkz[23] = 223670955L;
 note_inkz[24] = 230129536L;


 cur_pos = 0;
 // init the sine table
 for (i=0; i<256; i++) sineTable[i] = (char)(127.0 *sin(6.283*((float)i)/256.0));
 // tables used for keeping track of column contents
 for (i=0; i<NUM_REL_LINES;i++){
       lineDone[i] = 0;
       puckLine[i] = 0;
       noteLine[i] = 0;
 }

 // interrupt stuff
 // External interrupt for VSYNC and BURST
 EIMSK = (1<<INT0) | (1<<INT1); //enable external interrupts
 EICRA = (1<<ISC00)|(1<<ISC01)|(1<<ISC11); //falling edges for burst, rising edge for vsync
 linepos = 0;
 linecount = 0;

 // Enable interrupts
 sei();
}

int main(void)
{
 init();
 while(1){
       //before we start the first line, reset the flag
       if (linecount == (START_LINE - 1)){
               medFlag = 0;
       //lines of interest
       } else if (linecount >= START_LINE && linecount <= END_LINE && !lineDone[linecount-START_LINE]){
               //delay till the puck area
               _delay_us(11);
               //grab the result from analog comparator to see if puck is there
               puckLine[linecount-START_LINE] = (ACSR>>ACO);
               //delay again till we are at the note
               _delay_us(4);
               //grab result from AC to see if note is there
               noteLine[linecount-START_LINE] = (ACSR>>ACO);
               //mark the line as done so we don't do it twice
               lineDone[linecount-START_LINE] = 1;
       //after the last line, we process the data
       } else if (linecount == (END_LINE+1) && (medFlag == 0)){
               int temppuck = 0;
               int tempnote = 0;
               //find how many lines were bright for both columns
               for (i=0; i<NUM_REL_LINES;i++) {
                       lineDone[i] = 0;
                       temppuck += puckLine[i];
                       tempnote += noteLine[i];
               }
               //We try to find the median, so we divide by 2 to find where we should stop for the median
               temppuck = (temppuck + (temppuck%2))/2;
               tempnote = (tempnote + (tempnote%2))/2;
               //iterate through until we are at the median line and record what it is
               for (i=0; i<NUM_REL_LINES;i++){
                       if (puckLine[i])
                               if (!(--temppuck)) puckMed = i;
                       if (noteLine[i])
                               if (!(--tempnote)) noteMed = i;
               }
               //Find the difference between the median of where the puck and note are
               diff = puckMed - noteMed;
               //If the puck is above the note, play a higher note
               if (diff > 2) cur_pos = (cur_pos+1)%(NUM_FREQZ);
               //If the puck is below the note, play a lower note
               else if (diff < -2) cur_pos = cur_pos == 0 ? (NUM_FREQZ-1) : cur_pos -1;
               increment = note_inkz[cur_pos];
               medFlag++;
       }
 }
}
